@@ -4,6 +4,7 @@ import pandas as pd
 import time
 import joblib
 from .models import *
+from .cross_validation import *
 from .data_prepare import *
 from .encoders import *
 from pathlib import Path
@@ -122,7 +123,7 @@ class AutoML(object):
                 type_of_estimator=None, # classifier or regression
                 metric=None,
                 metric_round=4,
-                combined_score_opt=False,
+                combined_score_opt=True,
                 gpu=False, 
                 random_state=42
                 ):
@@ -149,59 +150,115 @@ class AutoML(object):
     def fit(self,
         X, 
         y,
-        timeout=200, # optimization time in seconds
+        timeout=300, # optimization time in seconds
         auto_parameters=True,
-        folds=10,
+        folds=5,
         score_folds=2,
         opt_lvl=2,
         early_stoping=100,
         feature_selection=True,
         verbose=1,
         ):
+        X_source = X.copy()
         ####################################################
-        # STEP 1
-        # DATA PREPROC
-        start_step_1 = time.time()
-
-        self.de = DataPrepare(
-            cat_encoder_names=['HelmertEncoder','CountEncoder','HashingEncoder'],
-            #outliers_threshold=3,
-            normalization=False,
-            random_state=self._random_state, 
-            verbose=verbose
-            )
-        X = self.de.fit_transform(X)
-
-        total_step_1 = (time.time() - start_step_1)
-
-        ####################################################
-        # STEP 2
-        # Model 1
-        start_step_2 = time.time()
+        # STEP 0
+        start_step_0 = time.time()
         if verbose > 0:
-            print('> Start Fit Base Models')
+            print('> Start Fit Base Model')
+
+        self.cat_features=X.columns[(X.dtypes == 'object') | (X.dtypes == 'category')]
+        X[self.cat_features] = X[self.cat_features].astype('str')
+        X.fillna(0, inplace=True)
+        X[self.cat_features] = X[self.cat_features].astype('category')
 
         self.model_1 = CatBoost(
             type_of_estimator=self.type_of_estimator, 
             random_state=self._random_state,
             gpu=self._gpu,
             )
-        self.model_1 = self.model_1.fit(X, y)
+        self.model_1 = self.model_1.fit(X, y, cat_features=self.cat_features.tolist())
+        X = None
+
+        ####################################################
+        # STEP 1
+        start_step_1 = time.time()
+
+        if verbose > 0:
+            print('> DATA PREPROC')
+        self.de_1 = DataPrepare(
+            cat_encoder_names=['OneHotEncoder', 'CountEncoder'],
+            #outliers_threshold=3,
+            normalization=True,
+            random_state=self._random_state, 
+            verbose=verbose
+            )
+        X = self.de_1.fit_transform(X_source,)
+        if self.de_1.cat_features is not None:
+            X = X.drop(self.de_1.cat_features, axis = 1)
+
+        if verbose > 0:
+            print('> Start Fit Models 1')
+        # Model 2
+        # self.model_2 = LinearModel(
+        #     type_of_estimator=self.type_of_estimator, 
+        #     random_state=self._random_state,
+        #     )
+        # self.model_2 = self.model_2.fit(X, y)
+
+        # Model 3
+        self.model_3 = MLP(
+            type_of_estimator=self.type_of_estimator, 
+            random_state=self._random_state
+            )
+        self.model_3 = self.model_3.fit(X, y)
+
+        X = None
+
+        total_step_1 = (time.time() - start_step_1)
+
+        ####################################################
+        # STEP 2
+        start_step_2 = time.time()
+
+        if verbose > 0:
+            print('> DATA PREPROC')
+
+        self.de_2 = DataPrepare(
+            cat_encoder_names=['HelmertEncoder','CountEncoder','HashingEncoder'],
+            #outliers_threshold=3,
+            normalization=False,
+            random_state=self._random_state, 
+            verbose=verbose
+            )
+        X = self.de_2.fit_transform(X_source)
+        #X = X.drop(self.de_2.cat_features, axis = 1)
+
+        if verbose > 0:
+            print('> Start Fit Models 2')
+
+        self.model_4 = CatBoost(
+            type_of_estimator=self.type_of_estimator, 
+            random_state=self._random_state,
+            gpu=self._gpu,
+            )
+
+        self.model_4 = self.model_4.fit(X, y)
 
         total_step_2 = (time.time() - start_step_2)
 
         ####################################################
         # STEP 3
-        # Model 4
+        # Model 2 - 3
         start_step_3 = time.time()
+
         if verbose > 0:
             print(50*'#')
             print('> Start Opt Model')
 
-        time_to_opt = (timeout - (time.time()-start_step_1)) - 60
+        time_to_opt = (timeout - (time.time()-start_step_0)) - 60
         time.sleep(0.1)
 
-        self.model_4 = LightGBM(
+        self.model_5 = LightGBM(
             type_of_estimator=self.type_of_estimator, 
             random_state=self._random_state,
             gpu=self._gpu,
@@ -219,48 +276,34 @@ class AutoML(object):
                 'early_stoping':early_stoping,
                 'feature_selection':feature_selection,
                 'timeout':time_to_opt, # optimization time in seconds
+                'iteration_check': False,
                 'verbose':verbose}
 
-        try:
-            history = self.model_4.opt(**params)
+        history = self.model_5.opt(**params)
             
-            self._select_features_model_4 = list(history['columns'].iloc[0].values)
-            self.best_params_model_4 = history['model_param'].iloc[0]
+        self._select_features_model_5 = list(history['columns'].iloc[0])
+        self.best_params_model_5 = history['model_param'].iloc[0]
+        folds=self.model_5._folds
 
-        except:
-            self._select_features_model_4 = list(X.columns.values)
-            self.best_params_model_4 = self.model_4._init_default_model_param()
-            print("Not enough time to find the optimal parameters. \n \
-                    Please, Increase the 'timeout' parameter for normal optimization.")
-
-        self.model_4 = LightGBM(
+        self.model_5 = LightGBM(
             type_of_estimator=self.type_of_estimator, 
             random_state=self._random_state,
             gpu=self._gpu,
             )
-        self.model_4.model_param = self.best_params_model_4
-        self.model_4 = self.model_4.fit(X[self._select_features_model_4], y, model_param=self.best_params_model_4)
+        self.model_5.model_param = self.best_params_model_5
 
-        total_step_3 = (time.time() - start_step_3)
+        self.cv_model_5 = CrossValidation(
+        estimator=self.model_5,
+        folds=folds,
+        n_repeats=1,
+        print_metric=False, 
+        random_state=self._random_state,
+        )
+        self.cv_model_5.fit(X[self._select_features_model_5], y,)
+
+        total_step_4 = (time.time() - start_step_3)
 
         ####################################################
-        # STEP 4
-        # Model 2 - 3
-        self.scaler = StandardScaler().fit(X)
-        X = self.scaler.transform(X)
-
-        # self.model_2 = LinearModel(
-        #     type_of_estimator=self.type_of_estimator, 
-        #     random_state=self._random_state,
-        #     )
-        # self.model_2 = self.model_2.fit(X, y)
-
-        # Model 3
-        self.model_3 = MLP(
-            type_of_estimator=self.type_of_estimator, 
-            random_state=self._random_state
-            )
-        self.model_3 = self.model_3.fit(X, y)
 
         if verbose > 0:
             print(50*'#')
@@ -279,41 +322,55 @@ class AutoML(object):
         if self.model_1 is None:
             raise Exception("No fit models")
 
+        X_source = X.copy()
+        ####################################################
+        # STEP 0
+        X[self.cat_features] = X[self.cat_features].astype('str')
+        X.fillna(0, inplace=True)
+        X[self.cat_features] = X[self.cat_features].astype('category')
+
+        self.predict_model_1 = self.model_1.predict_or_predict_proba(X)
+
         ####################################################
         # STEP 1
-        # DATA PREPROC
-        X = self.de.transform(X, verbose=verbose)
+        X = self.de_1.transform(X_source, verbose=verbose)
+        if self.de_1.cat_features is not None:
+            X = X.drop(self.de_1.cat_features, axis = 1)
+
+        #self.predict_model_2 = self.model_2.predict_or_predict_proba(X)
+        self.predict_model_3 = self.model_3.predict_or_predict_proba(X)
 
         ####################################################
         # STEP 2
-        # Models
-
-        if self.type_of_estimator == 'classifier':
-            predict_model_1 = self.model_1.predict_proba(X)
-            predict_model_4 = self.model_4.predict_proba(X[self._select_features_model_4])
-            X = self.scaler.transform(X)
-            predict_model_2 = self.model_2.predict_proba(X)
-            predict_model_3 = self.model_3.predict_proba(X)
-        else:
-            predict_model_1 = self.model_1.predict(X)
-            predict_model_4 = self.model_4.predict(X[self._select_features_model_4])
-            X = self.scaler.transform(X)
-            #predict_model_2 = self.model_2.predict(X)
-            predict_model_3 = self.model_3.predict(X)
+        X = self.de_2.transform(X_source, verbose=verbose)
+        #X = X.drop(self.de_2.cat_features, axis = 1)
 
+        self.predict_model_4 = self.model_4.predict_or_predict_proba(X)
+        
         ####################################################
         # STEP 3
+        self.predict_model_5 = self.cv_model_5.predict_test(X[self._select_features_model_5])
+        
+        ####################################################
+        # STEP 4
         # Blend
-        predicts = ((predict_model_1*2)+predict_model_3+predict_model_4)/4
-        return predicts
+        predicts = (
+            self.predict_model_1
+            #+self.predict_model_2
+            +self.predict_model_3
+            +self.predict_model_4
+            +self.predict_model_5)/4
+        return (predicts)
 
 
     def save(self, name='AutoML_dump', folder='./'):
         dir_tmp = folder+"AutoML_tmp/"
         Path(dir_tmp).mkdir(parents=True, exist_ok=True)
-        self.de.save(name='DataPrepare_dump', folder=dir_tmp)
+        self.de_1.save(name='DataPrepare_1_dump', folder=dir_tmp)
+        self.de_2.save(name='DataPrepare_2_dump', folder=dir_tmp)
         joblib.dump(self, dir_tmp+'AutoML'+'.pkl')
-        shutil.make_archive(dir+name, 'zip', dir_tmp)
+        self.cv_model_5.save(name='cv_model_5', folder=dir_tmp,)
+        shutil.make_archive(folder+name, 'zip', dir_tmp)
         shutil.rmtree(dir_tmp)
         print('Save AutoML')
 
@@ -321,10 +378,13 @@ class AutoML(object):
     def load(self, name='AutoML_dump', folder='./'):
         dir_tmp = folder+"AutoML_tmp/"
         Path(dir_tmp).mkdir(parents=True, exist_ok=True)
-        shutil.unpack_archive(dir+name+'.zip', dir_tmp)
+        shutil.unpack_archive(folder+name+'.zip', dir_tmp)
         model = joblib.load(dir_tmp+'AutoML'+'.pkl')
-        model.de = DataPrepare()
-        model.de = model.de.load('DataPrepare_dump', folder=dir_tmp)
+        model.de_1 = DataPrepare()
+        model.de_1 = model.de_1.load('DataPrepare_1_dump', folder=dir_tmp)
+        model.de_2 = DataPrepare()
+        model.de_2 = model.de_2.load('DataPrepare_2_dump', folder=dir_tmp)
+        model.cv_model_5 = model.cv_model_5.load(name='cv_model_5', folder=dir_tmp,)
         shutil.rmtree(dir_tmp)
         print('Load AutoML')
         return(model)
